/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("carter.ui.editors.DefinitionEditor");

// dojo dependencies:
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ComboBox");
dojo.require("dojo.parser");
dojo.require("dojo.io.iframe");
dojo.require("dojo._base.xhr");
dojo.require("dojo.i18n");

// Jazz/Kenton dependencies:
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.core.RootServices");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.FilterBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MessageBox");
dojo.require("rmps.core.error");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.util.ContextUtils");

// Carter dependencies:
dojo.require("carter.ui.editors.Definition");
dojo.require("carter.ui.editors.Domain");
dojo.require("carter.ui.editors.DefinitionsMgr");
dojo.requireLocalization("carter.ui", "EditorMessages");
dojo.require("carter.ui.utils");
dojo.require("carter.ui.error");

// Context help IDs:
var UAKEY_DEFINITION_PAGE = "com.ibm.rational.carter.web.page_DefinitionEditor01";
var UAKEY_GENERAL_SEC = "com.ibm.rational.carter.web.section_ImportDefinitionGeneralSection01";
var UAKEY_MODELS_SEC = "com.ibm.rational.carter.web.section_ImportDefinitionModelsSection01";
var UAKEY_SCHEDULE_SEC = "com.ibm.rational.carter.web.section_ImportDefinitionScheduleSection01";


// globals:
var messages = dojo.i18n.getLocalization("carter.ui", "EditorMessages");

var StreamClient = rmps.core.streams.StreamClient;

var Utils = carter.ui.utils;

var KEY_RETURN = 13;
var KEY_ENTER = 14;

var RENDER_DEF_NUM = "50";

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

// Editor decleration:
dojo.declare("carter.ui.editors.DefinitionEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "carter-ui-editors-DefinitionEditor",
	
	isNew: null,
	itemId: null,
	_definition: null,
	
	_widgetName: null,
	_widgetGeneral: null,
	_widgetModels: null,
	_widgetSchedule: null,
	_deferred: null,
	messages: null,
	_domains: null,
	_getDomains_deferred: null,

	constructor: function(arguments){
		// set context help id to the editor
		this.uaKey = UAKEY_DEFINITION_PAGE;
		
		this.messages = messages;
		if (arguments && arguments.editorInput) {	
			this.itemId = arguments.editorInput.itemId;
		}
		if( this.itemId == null || this.itemId == "new"){
			this.itemId = "new";
			this.isNew = true;
		}
		else {
			this.isNew = false;
		}
	},
	
	startup: function() {
		
		delete this._definition;
		this._definition = null;	
	},
/*******************************************************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2010.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*******************************************************************************/

    postCreate: function(args) {
        this.inherited(arguments);
        
        if( this._definition == null)
        	this._definition = new carter.ui.editors.Definition();
        else
        	this._definition.resetContent(); 
        
        this.showLoading();
        
        this._init();
       
        this.hideMessage();
        
        this._getAvailableDomainsFromServer();
    },
    
    _init: function() {
    	// Add hyperlink to definitions page:
    	if( this._headerContent ){
    		var link = dojo.create("a", { href: Utils.getDefinitionsViewUri(), innerHTML: messages.defintions_link});
    		this._headerContent.appendChild(link);	
    	}

    	this._createSections();		
	},
	
	_createSections: function() {
		this._createNameSection();
		this._createGeneralSection();
		this._createScheduleSection();
		this._createModelsListSection();
	},

	_createNameSection: function() {
		this._widgetName = new carter.ui.editors.DefinitionEditor._defName({editor: this});
		if( this._widgetName != null ) {
			this.containerNode.appendChild(this._widgetName.domNode);
			this._widgetName.startup();			
		}
	},
	
	_createGeneralSection: function() {
		this._widgetGeneral = new carter.ui.editors.DefinitionEditor._general({editor: this});
		if( this._widgetGeneral != null ) {
			this.containerNode.appendChild(this._widgetGeneral.domNode);
			this._widgetGeneral.startup();
			var projectId = jazz.app.context.get().itemId;
			this._widgetGeneral.getAvailableStreams(projectId);		
		}
	},

	_createModelsListSection: function() {
		this._widgetModels = new carter.ui.editors.DefinitionEditor._ModelsList({editor: this});
		if( this._widgetModels != null ) {
			this.containerNode.appendChild(this._widgetModels.domNode);
			this._widgetModels.startup();
		}	
	},

	_createScheduleSection: function() {
		this._widgetSchedule = new carter.ui.editors.DefinitionEditor._Schedule({editor: this});
		if( this._widgetSchedule != null ) {
			this.containerNode.appendChild(this._widgetSchedule.domNode);
			this._widgetSchedule.startup();
		}		
	},

	addButtons: function(toolbar) {		
		this.inherited(arguments);
		this.refreshButton.domNode.style.display = "none";
	},
	
	setDirty: function() {
		this.saveButton.setEnabled(true);
		this.hideMessage();
	},
	
	_getAvailableDomainsFromServer: function() {

		if( this._getDomains_deferred != null ) {
			this.loadContent(this.itemId);
			return;
		}

		var dataMgr = carter.ui.editors.DefinitionsMgr;
		if( dataMgr == null ) return;
			
		delete this._domains;
		
		
		this._getDomains_deferred = dataMgr.getDomains();
		
		if( this._getDomains_deferred != null ){
			
			this._getDomains_deferred.addCallback(this, function(domains) {
				if( domains != null )
					this._domains = dojo.clone(domains);
				else {
					this._domains = [];
					var domain = new carter.ui.editors.Domain();
					this._domains.push(domain);					
				}
				this.loadContent(this.itemId);
			});
			
			this._getDomains_deferred.addErrback(this, function(error) {
								
				var errMsg = carter.ui.error.getCarterError(this._getDomains_deferred.ioArgs);
				if( errMsg != null ){
					this.showErrorMsg(errMsg);
					return;
				}
				
				if (!dojo._isDocumentOk(this._getDomains_deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._getDomains_deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			
			this._getDomains_deferred.addBoth(this, function() {
				delete this._getDomains_deferred;
			});		
		}					
	},
	
	getDomains: function() {
		return this._domains;
	},
	
	save: function() {
		// validate user input:
		if( ! this._validateDataBeforeSave() ) return;
		
		if( this._definition == null )
			this._definition = new carter.ui.editors.Definition(); 
		else
			this._definition.resetContent();
				
		this._collectDefData(this._definition);			
		
		// save definition
		this.saveButton.setEnabled(false);
		
		var defMgr = carter.ui.editors.DefinitionsMgr;
		if( defMgr == null ) return;
		
		var def = dojo.clone(this._definition);
		this._deferred = defMgr.saveDefintion(def);
				
		if( this._deferred != null ){
			
			this._deferred.addCallback(this, function(definition) {
				
				
				var msg = this.messages.inf_DefinitionSavedOk ;
				var timeOut = 3000;
				
				if(!this._widgetModels || this._widgetModels.getModels().length == 0){
					msg += "," + this.messages.inf_ModelNeedConfigure;
					timeOut = 7000;
				}
				
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: msg,
					closable: true
				},timeOut);
				
				if( this._definition.defId == null || this._definition.defId == "new"){
					this._definition.defId = definition.defId;
					this.itemId = definition.defId;
				}
				
				this._definition.eTag = definition.eTag;
				
			});
			
			this._deferred.addErrback(this, function(error) {
				
				this.saveButton.setEnabled(true);
				
				var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
				if( errMsg != null ){
					this.showErrorMsg(errMsg);
					return;
				}
				
				if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
					carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
				} else {
					carter.ui.error.createError(error).dispatch();
				}
			});
			
			this._deferred.addBoth(this, function() {
				delete this._deferred;
			});		
		}			
	},
	
	showErrorMsg: function(msg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},

	_collectDefData: function(def) {
		
		def.defId = this.itemId;
		
		if( this._widgetName != null ) {
			def.name = this._widgetName.getName();
		}
		if( this._widgetGeneral != null ) {
			def.streamUri = this._widgetGeneral.getWorkspace();
			def.renderDiagrams = this._widgetGeneral.getRenderDiagrams();
			def.renderDiagramsThreshold = this._widgetGeneral.getRenderDiagramsThreshold();
			def.content = this._widgetGeneral.getContentLevel();
			def.fullImport = this._widgetGeneral.getFullImport();
			def.oneFullImport = this._widgetGeneral.isContentLevelChanged();
		}
		if( this._widgetModels != null ) {
			def.models = this._widgetModels.getModels();
		}
		if( this._widgetSchedule != null ) {
			def.schedule 			= this._widgetSchedule.isScheduleEnable();
			def.scheduleType 		= this._widgetSchedule.getScheduleType();
			def.scheduleInterval 	= this._widgetSchedule.getScheduleInterval();
			def.scheduleTime 		= this._widgetSchedule.getScheduleTime();
			def.scheduleDays 		= this._widgetSchedule.getSelectedDays();
		}
	},
	
	getSavedDefinition: function() {
		
		return this._definition;
	},
	
	loadContent: function(itemId) {
		
		this._resetContent();
		
		this.itemId = itemId;
			
		// get definition content from server...
		if( this.itemId != "new" ) {

			var defMgr = carter.ui.editors.DefinitionsMgr;
			if( defMgr != null ) {
				this._deferred = defMgr.getDefinition(this.itemId);
				if( this._deferred != null ){			
					this._deferred.addCallback(this, function(definition) {
						
						if( definition == null ) return;
						delete this._definition;
						this._definition = dojo.clone(definition);

						// populate the page:
						if( this._widgetName != null ) {
							this._widgetName.setName(this._definition.name);
						}
						if( this._widgetGeneral != null ) {
							this._widgetGeneral.setWorkspace(this._definition.streamUri);
							this._widgetGeneral.setRenderDiagram(this._definition.renderDiagrams);
							if (this._definition.renderDiagramsThreshold)
								this._widgetGeneral.setRenderDiagramsThreshold(this._definition.renderDiagramsThreshold);
							this._widgetGeneral.setContentLevel(this._definition.content);
							this._widgetGeneral.setFullImport(this._definition.fullImport);
						}
						if( this._widgetModels != null ) {
							this._widgetModels.addModels(this._definition.models);
						}
						if( this._widgetSchedule != null ) {
							this._widgetSchedule.setSchedule(this._definition);
						}
					});
					this._deferred.addErrback(this, function(error) {
						
						var errMsg = carter.ui.error.getCarterError(this._deferred.ioArgs);
						if( errMsg != null ){
							this.showErrorMsg(errMsg);
							return;
						}
						if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
							carter.ui.error.createXhrError(this._deferred.ioArgs).dispatch();
						} else {
							carter.ui.error.createError(error).dispatch();
						}
					});
					this._deferred.addBoth(this, function() {
						delete this._deferred;
					});
				}
			}
		}		
	},
	
	_resetContent: function() {
		
		this.itemId = "new";
		
		if( this._widgetName != null ) {
			this._widgetName.setName(this.messages.defaultDefinitionName);
		}
		if( this._widgetModels != null ) {
			this._widgetModels.resetContent();
		}
		if( this._widgetGeneral != null ) {
			this._widgetGeneral.resetContent();			
		}
		if( this._widgetSchedule != null ) {
			this._widgetSchedule.resetContent();			
		}
		
		this.hideMessage();
		this.saveButton.setEnabled(false);		 
	},
	
	_validateDataBeforeSave: function() {
	
		// check definition name:
		if( this._widgetName != null ) {
			var defName = this._widgetName.getName();
			if( defName.length == 0 ) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.err_emptyDefinitionName,
					closable: true
				});
				return false;
			}
		}
		
		// check render diagrams threshold
		if( this._widgetGeneral != null ) {
			if(this._widgetGeneral.isContentLevelChanged() && !confirm(messages.partialLevelWarning))
				return false;
			if( this._widgetGeneral.getRenderDiagrams() == true ) {
				var interval = this._widgetGeneral.getRenderDiagramsThreshold();
				var intRegex = /^\d+$/;
				if( interval == null || interval.length == 0 || !intRegex.test(interval)){
					var errMsg = ( interval == null || interval.length == 0 ) ? this.messages.err_noRenderDiagramsThreshold : this.messages.err_invalidRenderDiagramsThreshold; 
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: errMsg,
						closable: true
					});
					return false;			
				} 
			}
		}
		
		/*// check models
		if( this._widgetModels != null ) {
			
			var modelNames = this._widgetModels.getModelNames();
			
			// check for empty models list:
			if( modelNames == null || modelNames.length == 0 ) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.err_noDefinitionModels,
					closable: true
				});
				return false;
			}
				
			// check for empty model names:
			for(var i=0 ; i < modelNames.length ; i++ ) {
				var name = modelNames[i];
				if( name == null || name.length == 0 ) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: this.messages.err_noDefinitionModels,
						closable: true
					});
					return false;					
				}
			}
			// check for duplicated names:
			for(var j=0 ; j < (modelNames.length - 1) ; j++) {
				var name1 = modelNames[j];				
				for(var k = (j+1) ; k <= modelNames.length ; k++){
					var name2 = modelNames[k];
					if( name1 == name2){
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_DuplicatedModelNames,
							closable: true
						});
						return false;
					}
				}
			}
		}*/
		
		// check schedule
		if( this._widgetSchedule != null ) {
			if( this._widgetSchedule.isScheduleEnable() ){
				var days = this._widgetSchedule.getSelectedDays();
				if( days.length == 0) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: this.messages.err_noScheduleDay,
						closable: true
					});
					return false;					
				}
				
				if( this._widgetSchedule.getScheduleType() == 0 ) {
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_noScheduleType,
							closable: true
						});
						return false;								
				}
				
				if( this._widgetSchedule.getScheduleType() == SCHEDTYPE_INTERVAL ) {
					var interval = this._widgetSchedule.getScheduleInterval();
					if( interval == null || interval.length == 0){
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: this.messages.err_noScheduleInterval,
							closable: true
						});
						return false;			
					} 
				} 
			}
		}
		
		return true;
	}

});

// Definition name:
dojo.declare("carter.ui.editors.DefinitionEditor._defName", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("carter.ui", "internal/templates/DefinitionNameEditor.html"),
	widgetsInTemplate: true,
	dirty: false,
	editor: null,
	origName: null,
	messages: null,
	
	constructor: function(arguments){
		this.editor = arguments.editor;
		this.messages = messages; 
		this._defIconSrc = dojo.moduleUrl("carter.ui", "graphics/icons/definition.gif");
	},
	
    postCreate: function() {
		this.inherited(arguments);

		this._nameFieldHandlers = [];
		this._nameFieldHandlers.push(this.connect(this.nameEdit, "onkeypress", function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				dojo.stopEvent(e);
				if( this.origName != this.nameEdit.value ) {
					this.setDirty(true);
				}
				if( this.nameEdit.value == "") {
					this.setName(this.messages.defaultDefinitionName);
				}
				this.togglePrimaryField("Preview");
				
			}
		}));
		
		this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseover", function (e) {
			e.target.style.backgroundColor = "#FFFFCC";
		}));
		this._nameFieldHandlers.push(this.connect(this.previewName,"onmouseout", function (e) {
			e.target.style.backgroundColor = "";
		}));
		this._nameFieldHandlers.push(this.connect(this.previewName,"onfocus", function (e) {
			this.togglePrimaryField("Edit");
		}));
		this._nameFieldHandlers.push(this.connect(this.nameEdit, "onblur", function (e) {
			this.togglePrimaryField("Preview");
		}));
		
		this.connect(this.nameEdit,"onchange", function (e) {
			this.setDirty(true);
		});
		
		this.setName(this.messages.defaultDefinitionName);
		this.togglePrimaryField("Preview");		
	},

	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.nameEdit.style.display = "none";
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.nameEdit.value || this.messages.defaultDefinitionName));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.nameEdit.style.display = "";
			this.nameEdit.focus();
			this.nameEdit.select();
			if (this.header)
				this.header.hideMessage();
		}
	},
	
	setName: function(name) {
			this.origName = name;
			this.nameEdit.value = name;
			this.previewName.innerHTML = "";
			this.previewName.appendChild(document.createTextNode(this.nameEdit.value));		
	},
	
	getName: function() {
		return this.nameEdit.value;	
	},
	
	setDirty: function (dirty) {
		
		if (dirty) {
			if( this.editor != null ) {
				this.editor.setDirty();
			}
			//this.modifiedFlag.style.display = "inline";
		} 
		else {
			//this.modifiedFlag.style.display = "none";
		}
		this.dirty = dirty;
	}	
});

// General attributes:
dojo.declare("carter.ui.editors.DefinitionEditor._general", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div>' +
			'<p>&nbsp;</p>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" readOnly="${editor.isReadOnly}" collapsible="false" uaKey="${uaKey}" primaryTitle="${messages.general}"">' +
				'<table class="general">' +
					'<tbody dojoAttachPoint="_tbody">' +
						'<tr dojoAttachPoint="_renderDiagramsSection">' + 
							'<td colspan="4">' +
								'<br>' +
								'<input dojoAttachPoint="_enabledRD" type="checkbox" dojoAttachEvent="onclick: _onEnabledRenderedDiagrams">${messages.renderDiagrams}' +
								'<input dojoAttachPoint="_ElementsThresholdInput" type="text" maxlength="4" style="width: 50px;">' +
							'</td>' +
						'</tr>' +
					'</tr>' +
					'</tbody>' +
				'</table>' +
			'</div>' +
		'</div>',

	widgetsInTemplate: true,
	editor: null,
	_selectWorkspace: null,
	_streamUri: null,
	_defaultStreamUri: null,
	
	_selectContent: null,
	_content: null,
	_currentCotentLevel: null,
	
	messages: messages,
	uaKey: UAKEY_GENERAL_SEC,
	
	constructor: function(arguments){
		this.editor = arguments.editor;
		this.messages = messages; 
	},
	
    postCreate: function() {
		this.inherited(arguments); 
		this._loadContent();
		// Add event handlers:
		connectOnInput(this._ElementsThresholdInput, this, function() {
			this._onThresholdChanged();
        });
	},
	
	resetContent: function() {
		
		this._enabledRD.checked = false;
		this.setRenderDiagramsThreshold(RENDER_DEF_NUM);
		this._ElementsThresholdInput.disabled = true;
		this._content = Utils.CONTENT_LEVEL_DEFAULT;
		this._currentCotentLevel = this._content;
		this._enableFilter.checked = false;
		this._selectContent.disabled = true;
		this._enableReImport.checked = false;
	},
	
	_loadContent: function() {
		
		var tr = dojo.create("tr");
		
		// add workspace label column
		var labelTd = dojo.create("td", {width: "120px"}, tr);
		var labelContainer = dojo.create("div", null, labelTd);
		var labelNode = dojo.create("span", null, labelContainer);
		
		var cshNode = dojo.create("span", null, labelNode);
		
		cshNode.appendChild(dojo.doc.createTextNode(this.messages.workspace));
		
		// add workspace selection:
		var valueTd = dojo.create("td", null, tr);
		var container = dojo.create("div", null, valueTd);
		
		this._selectWorkspace = dojo.doc.createElement("select", {"id":"workspace"});
		this._selectWorkspace.id= "workspace";
		this.connect(this._selectWorkspace, "onchange", this._onStreamChange);
		container.appendChild(this._selectWorkspace);
		this._tbody.insertBefore(tr, this._renderDiagramsSection);
        
        // Initialize stream uri value:
        if( ! this._defaultStreamUri ) {
			var defaultStreamDeferred = StreamClient.getDefaultStream();
			defaultStreamDeferred.addCallback(dojo.hitch(this, "_getDefaultStreamCallback"));
        }
        
        if(! this._streamUri ){
        	var contextUtils = rmps.core.util.ContextUtils;
        	var tempUri = contextUtils.getCurrentContextURI();
        	if(contextUtils.getContextType(tempUri) == rmps.core.util.ContextTypes.WORKSPACE) 
        		this._streamUri = tempUri;
        	else if(this._defaultStreamUri)
        		this._streamUri = this._defaultStreamUri;
        }
        // add content label
        tr = dojo.create("tr", {style: "height:60px"});
        td = dojo.create("td", {width: "200px"}, tr);
        
        this._enableFilter = dojo.create("input" ,{"type":"checkbox"}, td);
		this.connect(this._enableFilter, "onclick", this._onEnableFilter);
		
		td.appendChild(dojo.doc.createTextNode(this.messages.content));
        
		// add content select
		td = dojo.create("td", null, tr);
		this._selectContent = dojo.create("select", {"id":"content"}, td);
		this.connect(this._selectContent, "onchange", this._onContentLevelChange);
		var option = dojo.create("option", {selected: (Utils.CONTENT_LEVEL_DEFAULT == Utils.CONTENT_LEVEL_HIGH)}, this._selectContent);
		option.appendChild(dojo.doc.createTextNode(Utils.CONTENT_LEVEL_HIGH));
		option = dojo.create("option", {selected: (Utils.CONTENT_LEVEL_DEFAULT == Utils.CONTENT_LEVEL_MEDIUM)}, this._selectContent);
		option.appendChild(dojo.doc.createTextNode(Utils.CONTENT_LEVEL_MEDIUM));
		option = dojo.create("option", {"selected": (Utils.CONTENT_LEVEL_DEFAULT == Utils.CONTENT_LEVEL_LOW)}, this._selectContent);
		option.appendChild(dojo.doc.createTextNode(Utils.CONTENT_LEVEL_LOW));
		//types text area
		td = dojo.create("td", null, tr);
		td.appendChild(dojo.doc.createTextNode(this.messages.types));
		td = dojo.create("td", {colspan: "2", style: "width:40%"}, tr);
		this._typesTextArea = dojo.create("textarea", {id:"shortLinkText", readOnly:"true" , style: "width:100%;height:7em;"}, td);
		this._tbody.insertBefore(tr, this._renderDiagramsSection);
		
		//re-import
		tr = dojo.create("tr");
		td = dojo.create("td", {colspan: "2"}, tr);
		this._enableReImport = dojo.create("input" ,{"type":"checkbox"}, td);
		this.connect(this._enableReImport, "onclick", this._onEnableReImport);
		td.appendChild(dojo.doc.createTextNode(this.messages.reImport));
		this._tbody.insertBefore(tr, this._renderDiagramsSection);
	},
	
	_updateTextAreaByContentLevel: function(contentLevel){
		var dataMgr = carter.ui.editors.DefinitionsMgr;
		if( dataMgr == null ) return;
		
		var deffered = dataMgr.getTypesByLevel(contentLevel);
		if( deffered != null ){			
			deffered.addCallback(this, function(types) {
				if( types != null ) {
					types = types.replace(/,/g, '\n');
					this._typesTextArea.value = types;
				}
			});
		}
	},
	
	isContentLevelChanged: function() {
		return (this._content != this._currentCotentLevel);
	},
	
	_onContentLevelChange: function() {
		this._content = this._selectContent.options[this._selectContent.selectedIndex].value;
		this._updateTextAreaByContentLevel(this._content);
		this.setDirty();
	},
	
	_onEnableReImport:function() {
		this.setDirty();
	},
	
	getFullImport: function(){
		return this._enableReImport.checked;
	},
	
	setFullImport: function(enableFullImport) {
		this._enableReImport.checked = enableFullImport;
	},
	
	setContentLevel: function(contentLevel) {
		if(!contentLevel) {
			contentLevel = Utils.CONTENT_LEVEL_DEFAULT;
		}
		this._content = contentLevel;
		this._currentCotentLevel = this._content; 
		if(this._content == Utils.CONTENT_LEVEL_FULL) {
			this._enableFilter.checked = false;
		}
		else {
			this._enableFilter.checked = true;
			this._updateSelectContent(contentLevel);
		}
		this._updateSelectContentAndTextArea();
	},
	
	_updateSelectContentAndTextArea :function(){
		this._selectContent.disabled = !this._enableFilter.checked;
		this._typesTextArea.disabled = this._selectContent.disabled; 
		if(this._typesTextArea.disabled) {
			this._typesTextArea.value = "";
			this._content = Utils.CONTENT_LEVEL_FULL; 
		}
		else {
			this._content = this._selectContent.options[this._selectContent.selectedIndex].value;
			this._updateTextAreaByContentLevel(this._content);
		}
	},
	
	_onEnableFilter: function() {
		this._updateSelectContentAndTextArea();
		this.setDirty();
	},
	
	getContentLevel: function() {
		if( this._content != null ) {
			return this._content;
		}
		return Utils.CONTENT_LEVEL_DEFAULT;
	},
	
	_updateSelectContent: function(contentLevel) {
		if( contentLevel && this._selectContent ){
			var size = this._selectContent.options.length;
			for (var i = 0; i < this._selectContent.options.length; i++) {
				if (contentLevel == this._selectContent.options[i].value ) {
					this._selectContent.selectedIndex = i;
					this._content = contentLevel;
				}
			}			
		}	
	},
	
	_onStreamChange: function() {
		this._streamUri = this._selectWorkspace.options[this._selectWorkspace.selectedIndex].value;		
		this.setDirty();
	},

	getAvailableStreams: function(projectId){
		var streamsDeferred = StreamClient.getStreams(projectId);
        streamsDeferred.addCallback(dojo.hitch(this, "_getAvailableStreamsCallback"));
	},
	
	_getAvailableStreamsCallback: function(streams){
		if( this._selectWorkspace != null ) {
			for (var i = 0; i < streams.length; i++) {
				var option = dojo.doc.createElement("option");
				option.value = streams[i].uri;
				option.appendChild(dojo.doc.createTextNode(streams[i].name));
				this._selectWorkspace.appendChild(option);
				if (this._streamUri == option.value) {
					this._selectWorkspace.selectedIndex = i;
					this._originalStreamUri = option.value;
				}
			}
		}
		if(this.editor._definition != null && this.editor._definition.streamUri != null) {
			this.setWorkspace(this.editor._definition.streamUri);
		}
	},
	
	_getDefaultStreamCallback: function(streamUri) {
		this._defaultStreamUri = streamUri;
		if(this._streamUri == null) {
			this.setWorkspace(this._defaultStreamUri);
		}
	},
	
	getRenderDiagrams: function() {
		return this._enabledRD.checked;
	},

	setRenderDiagram: function(enableRD) {
		this._enabledRD.checked = enableRD;
		this._ElementsThresholdInput.disabled = !enableRD;
	},
	
	_onEnabledRenderedDiagrams:function() {
		this._ElementsThresholdInput.disabled = !this._enabledRD.checked;
		this.setDirty();
	},
	
	getRenderDiagramsThreshold: function() {
		return this._ElementsThresholdInput.value;
	},

	setRenderDiagramsThreshold: function(threshold) {
		if (threshold)
			this._ElementsThresholdInput.value = threshold;
	},
	
	_onThresholdChanged: function() {
		this.setDirty();
	},
		
	getWorkspace: function() {
		if( this._streamUri != null ) {
			return this._streamUri;
		}
		return this._defaultStreamUri;	
	},
	
	setWorkspace: function(streamUri) {
		if( ! streamUri ) {
			streamUri = this._defaultStreamUri;
		}
		if( streamUri && this._selectWorkspace ){
			var size = this._selectWorkspace.options.length;
			for (var i = 0; i < this._selectWorkspace.options.length; i++) {
				if (streamUri == this._selectWorkspace.options[i].value ) {
					this._selectWorkspace.selectedIndex = i;
					this._streamUri = streamUri;
				}
			}			
		}	
	},

	setDirty: function () {
		if( this.editor != null ) {
			this.editor.setDirty();
		}
	}	
});

// Models list:
dojo.declare("carter.ui.editors.DefinitionEditor._ModelsList", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	editor: null,
	messages: messages,
	uaKey: UAKEY_MODELS_SEC,
	
	templateString: '<div>' +
		'<p>&nbsp;</p>' + 
		'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="true" isCollapsed="true" uaKey="${uaKey}"></div>' +
		'</div>',

	constructor: function(arguments){
		this.editor = arguments.editor;
	},
		
    postCreate: function() {
		this.inherited(arguments); 
		this._loadContent();
	},
	
	_loadContent: function() {
				
		// Pending on previous load:
		if (this._deferred) {
			return;
		}
				
		// Clear previous content...
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(this.messages.defModels));
		this._section.setPrimaryTitle(titleNode);
		
		var secondaryTitle = dojo.create("span");
		secondaryTitle.appendChild(dojo.doc.createTextNode(this.messages.inf_ModelNeedConfigure));
		this._section.setSecondaryTitle(secondaryTitle);
		
		//create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: this.messages.modelName, width: "70%"},
					{label: this.messages.domain, width: "30%"}
				]
		});
		this._table.domNode.style.width = "100%";
		this._table.domNode.style.overflwo = "auto";
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
							
	},
		
	_addDefModel: function(data) {
		var mName 		= (data != null)? data.name : this.messages.defaultModelName;
		var domainName 	= (data != null)? data.domainName : Utils.DOMAINNAME_RHAPSODY;
		var newModel 	= (data != null)? false : true;
		
		var row = new carter.ui.editors.DefinitionEditor._ModelRow(
			{editor: this.editor, 
			 modelName: mName, 
			 domainName: domainName,
			 newItem: newModel,
			 list: this});

		if( data == null ) {
			this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		}
		else {
			this._table.tbody.appendChild(row.domNode);
		}

		if(! data ) {
			row.focus();
			this.editor.setDirty();
		}
	},
	
	addModels: function(models) {
		if( models != null ) {
			for(var idx = 0 ; idx < models.length ; idx ++ ) {
				this._addDefModel(models[idx]);
			}
		}
	},
	
	resetContent: function() {
	
		var count = 0;
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			row.destroyRecursive();
			delete row;
			count += 1;
		});
		
		dojox.xml.parser.removeChildren(this._table.tbody);
	},
	
	getModels: function() {
		var models = [];	
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			var model = new carter.ui.editors.Model();
			model.name = row.getName();
			model.domainName = row.getDomain();
			models.push(model);
		});
		
		return models;
	},
	
	getModelNames: function() {
		var modelNames = [];	
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) {
			modelNames.push(row.getName());
		});
		
		return modelNames;
	}

});

dojo.declare("carter.ui.editors.DefinitionEditor._ModelRow", [dijit._Widget, dijit._Templated], {

    widgetsInTemplate: true,
    templateString: '<tr>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_modelNamePreview"></span>' +
							'</div>' +
						'</td>' +
						'<td>' +
							'<div>' +
								'<span class="preview" dojoAttachPoint="_domainPreview"></span>' +
							'</div>' +
						'</td>' +
					'</tr>',

    messages: messages,
    editor: null,

    _modelName: null,
    _deleteModelDeferred: null,

    _popupContent: null,
    _popup: null,
    _actionBar: null,
    _newItem: null,
    _list: null,
    _domain: null,

    constructor: function(arguments) {
        this.editor = arguments.editor;
        this._modelName = arguments.modelName;
        this._newItem = arguments.newItem;
        this._list = arguments.list;
        this._domain = arguments.domainName;
    },

    postCreate: function() {
    	var isNewRow = true;
        this._modelNamePreview.innerHTML = "";
		this._modelNamePreview.appendChild(dojo.doc.createTextNode(this._modelName));
		
		this._domainPreview.innerHTML = "";
		
		// set domain name 
		if( this.editor != null ){
			var domains = this.editor.getDomains();
			if( domains != null ){
				for (var i = 0; i < domains.length; ++i) {
					var domain = domains[i];
					if( this._domain == domain.internalName ){
						this._domainPreview.appendChild(dojo.doc.createTextNode(domain.displayName));
						break;
					}
					
				}						
			}
		}
		
		if( this._newItem && this._newItem == true ) {
        	this._modelNameEdit.focus();
        }
    },
	
    focus: function() {
        this._modelNameEdit.focus();
    },

    setName: function(name) {
        this._modelName = name;
        this._modelNameEdit.value = this._modelName; 
    },
	
	getName: function() {
		return this._modelName; 
	},
	
	getDomain: function() {
		if( this._domain == null )
			this._domain = this._domainSelect.options[this._domainSelect.selectedIndex].value;
		return this._domain;
	},
	
    destroy: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
            delete this._popupContent;
        }
        if (this._popup) {
            this._popup.destroyRecursive();
            delete this._popup;
        }
        this.inherited(arguments);
    },

    _deleteModel: function() {
		if( this.editor != null ) {
			this.editor.setDirty();
		}
    },

    _updateActions: function() {

        this._actionBar.addButton(this, function(e) {
            dojo.stopEvent(e);
            this._deleteModel();
            this.destroyRecursive();
            if (this.editor != null)
                this.editor.setDirty();
        }, {
            label: messages.msgDelete,
            imgSrc: dojo.moduleUrl("carter.ui", "graphics/icons/remove.gif")
        });
    },

    _createPopupContents: function() {
        if (this._popupContent) {
            this._popupContent.destroyRecursive();
        }
    },    
});

// Schedule widget:
dojo.declare("carter.ui.editors.DefinitionEditor._Schedule", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	editor: null,
	_section: null,
	_definition: null,
	_modifiedDefinition: null,
	messages: messages,
	uaKey: UAKEY_SCHEDULE_SEC,

	templateString: '<div><p>&nbsp;</p>' +
						'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible="false" uaKey="${uaKey}" primaryTitle="${messages.schedule}"">' +
							'<div class="schedule">' +
								'<div class="item">' +
									'<br>' +
									'<input id="${id}_eCb" type="checkbox" dojoAttachPoint="_enabledCb" dojoAttachEvent="onclick: _onEnabled">' +
									'<label for="${id}_eCb">${messages.enableSchedule}</label>' +
									'<br><br>' +
								'</div>' +
								'<fieldset style="float: left;">' +
									'<legend>${messages.importTime}</legend>' +
									'<br>' +
										'<div class="item">' +
											'<input dojoAttachPoint="_intervalRadio" type="radio" name="scheduleType" value="interval" dojoAttachEvent="onclick: _onInterval"> ${messages.interval} ' +
											'<input dojoAttachPoint="_intervalInput" type="text" maxlength="4" style="width: 50px;">' +
											'<br><br>' +
											'<input dojoAttachPoint="_specificRadio" type="radio" name="scheduleType" value="specific" dojoAttachEvent="onclick: _onSpecific"> ${messages.at} ' +
											'<select dojoAttachPoint="_hoursSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> : ' +
											'<select dojoAttachPoint="_minutesSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ' +
											'<select dojoAttachPoint="_ampmSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ${messages.gmt}' + 
										'</div>' +
								'<br><br>' +
								'</fieldset>' +
								'<fieldset style="float: left; margin-left: 20px;">' +
									'<legend>${messages.importDays}</legend>' +
									'<div class="item">' +
										'<select dojoAttachPoint="_daysSelect" dojoAttachEvent="onchange: _onDaysChanged" multiple="true" size="7" style="margin-right: 10px; overflow: hidden; float: left;"></select>' +
										'<div style="float: left;">' +
											'<button dojoAttachPoint="_selectAllBtn" style="width: 100%; margin-bottom: 4px;" dojoAttachEvent="onclick: _onSelectAllDays">${messages.selectAll}</button><br>' +
											'<button dojoAttachPoint="_deselectAllBtn" style="width: 100%;" dojoAttachEvent="onclick: _onDeselectAllDays">${messages.deselectAll}</button>' +
										'</div>' +
										'<div style="clear: both;"></div>' + // clearing div
									'</div>' +
								'</fieldset>' +
								'<div style="clear: both;"></div>' + // clearing div
							'</div>' +
						'</div>' +
					'</div>',

	
	postCreate: function() {
		if (dojo.isIE < 8) {
			this._selectAllBtn.style.width = "";
			this._deselectAllBtn.style.width = "";
		}
		
		if( this.editor == null ) return;
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(this.messages.schedule));				
		this._section.setPrimaryTitle(titleNode);
		
		this._definition = this.editor._definition;
		this._ModifiedDefinition = dojo.clone(this._definition);
		

		// populate hours
		for (var i = 1; i <= 12; ++i) {
			var label = i + "";
			if (i < 10) {
			 	label = "0" + i;
			}
			var value = label;
			if (i === 12) {
				value = "00";
			}
			this._createOption(this._hoursSelect, label, value, false);
		}
		
		// populate minutes
		for (var i = 0; i < 60; ++i) {
			var value = i;
			if (i < 10) {
			 	value = "0" + i;
			}
			this._createOption(this._minutesSelect, value, value, false);
		}
		
		// populate AM / PM
		var isPM = false;
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["dayPeriods-format-wide-am"], "am", !isPM);
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["dayPeriods-format-wide-pm"], "pm", isPM);
		
		// populate week days
		var weekDays = dojo.date.locale.getNames("days", "wide");
		
		// move sunday to the end
		weekDays.push(weekDays[0]);
		weekDays.shift();
		
		for (var i = 0; i < weekDays.length; ++i) {
			this._createOption(this._daysSelect, weekDays[i], i + 1, false);
		}
		
		// disable schedule:
		this._updateEnablement( false );
		
		// Add event handlers:
		connectOnInput(this._intervalInput, this, function() {
			this._onIntervalChanged();
        });
		
	},
	
	startup: function() {

		this.inherited(arguments);
		if (dojo.isIE < 8) {
			var sow = this._selectAllBtn.offsetWidth;
			var dow = this._deselectAllBtn.offsetWidth;
			if (sow < dow) {
				this._selectAllBtn.style.width = dow + "px";
			} 
			else {
				this._deselectAllBtn.style.width = sow + "px";
			}
		}

	},
	
	_createOption: function(selectNode, label, value, selected) {

		var option = dojo.create("option");
		option.value = value;
		option.appendChild(dojo.doc.createTextNode(label));
		if (selected) {
			option.selected = true;
		}
		selectNode.appendChild(option);

	},
	
	_updateEnablement: function(enabled) {

		if (enabled) {
			this._section.setSecondaryTitle(messages.scheduleEnabled);
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.removeClass(option, "disabledSelected");
					}
				}
			}
			
			this._intervalRadio.disabled = false;
			this._specificRadio.disabled = false;
			this._daysSelect.disabled = false;
			this._selectAllBtn.disabled = false;
			this._deselectAllBtn.disabled = false;
			
			if (this._intervalRadio.checked) {
				this._intervalInput.disabled = false;
				this._hoursSelect.disabled = true;
				this._minutesSelect.disabled = true;
				this._ampmSelect.disabled = true;
			} else {
				this._intervalInput.disabled = true;
				this._hoursSelect.disabled = false;
				this._minutesSelect.disabled = false;
				this._ampmSelect.disabled = false;
			}
			var TimeSelected = (this._intervalRadio.checked || this._specificRadio.checked)? true : false;
			if( ! TimeSelected ){
				this._specificRadio.checked = true;
				this._hoursSelect.disabled = false;
				this._minutesSelect.disabled = false;
				this._ampmSelect.disabled = false;			
			}
		} else {
			this._section.setSecondaryTitle(messages.scheduleDisabled);
			this._intervalRadio.disabled = true;
			this._specificRadio.disabled = true;
			this._intervalInput.disabled = true;
			this._hoursSelect.disabled = true;
			this._minutesSelect.disabled = true;
			this._ampmSelect.disabled = true;
			this._daysSelect.disabled = true;
			this._selectAllBtn.disabled = true;
			this._deselectAllBtn.disabled = true;
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.addClass(option, "disabledSelected");
					}
				}
			}
		}
	},
	
	_onEnabled: function() {		

		this._updateEnablement(this._enabledCb.checked);
		this.editor.setDirty();
	},
	
	_onInterval: function() {
		this._updateEnablement(true);
		this.editor.setDirty();
	},
	
	_onSpecific: function() {
		this._updateEnablement(true);
		this.editor.setDirty();
	},
	
	_onSelectAllDays: function() {

		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = true;
		}
		this._onDaysChanged();

	},
	
	_onDeselectAllDays: function() {

		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		this._onDaysChanged();

	},
	
	_onIntervalChanged: function() {
		this.editor.setDirty();
	},
	
	_onScheduleTimeChanged: function() {
		this.editor.setDirty();
	},
		
	_onDaysChanged: function() {				
		this.editor.setDirty();
	},
	
	_parseHour: function(hour) {
		return parseInt(hour.replace(/^0/, ''));
	},
	
	resetContent: function() {
		
		// deselect days:
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		
		this._enabledCb.checked = false;
		this._updateEnablement(false);
		
	},
	
	setSchedule: function(definition) {
		
		this.resetContent();
		
		if( definition == null ) return;
		if( definition.schedule == SCHED_DISABLE ) return;
		
		
		if (definition.scheduleType == SCHEDTYPE_INTERVAL ) {
			this._intervalRadio.checked = true;
		}
		else if( definition.scheduleType == SCHEDTYPE_FIXED ) {
			this._specificRadio.checked = true;
		}
		
		this._intervalInput.value = definition.scheduleInterval;
		

		//	********* Select time ************* 
		var nHour = parseInt(definition.scheduleTime[0]);
		var isPm = false;

		if( nHour > 12 ){
			isPm = true;
			nHour -= 12;
		}
		
		var amPmItemId = (isPm)? 1 : 0;
		var amPmItem = this._ampmSelect.options[amPmItemId];
		amPmItem.selected = true;
		
		var option = this._hoursSelect.options[nHour - 1];
		if( option != null ){
			option.selected = true;
		}

		var nMin = parseInt(definition.scheduleTime[1]);		
		var option = this._minutesSelect.options[nMin];
		if( option != null ){
			option.selected = true;
		}

		//	********* Select days *************
		// deselect all days first 
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		
		for( var i = 0 ; i < definition.scheduleDays.length ; i++ ) {
			var day = definition.scheduleDays[i];
			if( day > 0 && day <= this._daysSelect.options.length ) {
				var option = this._daysSelect.options[day-1];
				if ( option != null )
					option.selected = true;
			} 
		}
		
		this._enabledCb.checked = definition.schedule;
		
		this._updateEnablement(this._enabledCb.checked);		
	},
	
	getSelectedDays: function() {
		var days = [];
		var options = this._daysSelect.options;
		for (var i = 0, option ; option = options[i] ; ++i) {
			if (option != null && option.selected) {
				days.push(i + 1);
			}
		}
		return days;
	},
	
	isScheduleEnable: function() {
		if( this._enabledCb.checked )
			return SCHED_ENABLE;
		else 
			return SCHED_DISABLE; 
	},
	
	getScheduleType: function() {
		if( this._intervalRadio.checked ) 		return SCHEDTYPE_INTERVAL;
		else if( this._specificRadio.checked ) 	return SCHEDTYPE_FIXED;
		else 									return 0;
	},
	
	getScheduleInterval: function() {
		return this._intervalInput.value;
	},
	
	getScheduleTime: function() {
		var hour = this._hoursSelect.options[this._hoursSelect.selectedIndex].value;
		var minutes = this._minutesSelect.options[this._minutesSelect.selectedIndex].value;
		if (this._ampmSelect.options[this._ampmSelect.selectedIndex].value === "pm") {
			hour = this._parseHour(hour) + 12;
		}

		var time = [hour, minutes];
		return time;
	}
});

